@@ -4,11 +4,19 @@ import requests
 
 from pywink.devices import types as device_types
 from pywink.devices.factory import build_device
+from pywink.devices.standard import WinkPorkfolioNose
 from pywink.devices.sensors import WinkSensorPod, WinkHumiditySensor, WinkBrightnessSensor, WinkSoundPresenceSensor, \
-    WinkTemperatureSensor, WinkVibrationPresenceSensor, WinkLiquidPresenceSensor
+    WinkTemperatureSensor, WinkVibrationPresenceSensor, \
+    WinkLiquidPresenceSensor, WinkCurrencySensor, WinkMotionSensor, \
+    WinkPresenceSensor, WinkProximitySensor, WinkSmokeDetector, \
+    WinkCoDetector, WinkHub
 from pywink.devices.types import DEVICE_ID_KEYS
 
 API_HEADERS = {}
+CLIENT_ID = None
+CLIENT_SECRET = None
+REFRESH_TOKEN = None
+USER_AGENT = None
 
 
 class WinkApiInterface(object):
@@ -29,6 +37,14 @@ class WinkApiInterface(object):
         arequest = requests.put(url_string,
                                 data=json.dumps(state),
                                 headers=API_HEADERS)
+        if arequest.status_code == 401:
+            new_token = refresh_access_token()
+            if new_token:
+                arequest = requests.put(url_string,
+                                        data=json.dumps(state),
+                                        headers=API_HEADERS)
+            else:
+                raise WinkAPIException("Failed to refresh access token.")
         return arequest.json()
 
     def get_device_state(self, device, id_override=None):
@@ -42,6 +58,14 @@ class WinkApiInterface(object):
         return arequest.json()
 
 
+def get_set_access_token():
+    auth = API_HEADERS.get("Authorization")
+    if auth is not None:
+        return auth.split()[1]
+    else:
+        return None
+
+
 def set_bearer_token(token):
     global API_HEADERS
 
@@ -49,14 +73,28 @@ def set_bearer_token(token):
         "Content-Type": "application/json",
         "Authorization": "Bearer {}".format(token)
     }
+    if USER_AGENT:
+        API_HEADERS["User-Agent"] = USER_AGENT
 
 
-def set_wink_credentials(client_id, client_secret, refresh_token):
+def set_user_agent(user_agent):
+    global USER_AGENT
+
+    USER_AGENT = user_agent
+
+
+def set_wink_credentials(email, password, client_id, client_secret):
+    global CLIENT_ID, CLIENT_SECRET, REFRESH_TOKEN
+
+    CLIENT_ID = client_id
+    CLIENT_SECRET = client_secret
+
     data = {
         "client_id": client_id,
         "client_secret": client_secret,
-        "grant_type": "refresh_token",
-        "refresh_token": refresh_token
+        "grant_type": "password",
+        "email": email,
+        "password": password
     }
     headers = {
         'Content-Type': 'application/json'
@@ -66,8 +104,30 @@ def set_wink_credentials(client_id, client_secret, refresh_token):
                              headers=headers)
     response_json = response.json()
     access_token = response_json.get('access_token')
+    REFRESH_TOKEN = response_json.get('refresh_token')
     set_bearer_token(access_token)
-    return access_token
+
+
+def refresh_access_token():
+    if CLIENT_ID and CLIENT_SECRET and REFRESH_TOKEN:
+        data = {
+            "client_id": CLIENT_ID,
+            "client_secret": CLIENT_SECRET,
+            "grant_type": "refresh_token",
+            "refresh_token": REFRESH_TOKEN
+        }
+        headers = {
+            'Content-Type': 'application/json'
+        }
+        response = requests.post('{}/oauth2/token'.format(WinkApiInterface.BASE_URL),
+                                 data=json.dumps(data),
+                                 headers=headers)
+        response_json = response.json()
+        access_token = response_json.get('access_token')
+        set_bearer_token(access_token)
+        return access_token
+    else:
+        return None
 
 
 def get_bulbs():
@@ -110,6 +170,26 @@ def get_keys():
     return get_devices(device_types.KEY)
 
 
+def get_piggy_banks():
+    return get_devices(device_types.PIGGY_BANK)
+
+
+def get_smoke_and_co_detectors():
+    return get_devices(device_types.SMOKE_DETECTOR)
+
+
+def get_thermostats():
+    return get_devices(device_types.THERMOSTAT)
+
+
+def get_hubs():
+    return get_devices(device_types.HUB)
+
+
+def get_fans():
+    return get_devices(device_types.FAN)
+
+
 def get_subscription_key():
     response_dict = wink_api_fetch()
     first_device = response_dict.get('data')[0]
@@ -149,36 +229,54 @@ def get_devices_from_response_dict(response_dict, filter_key):
 
     devices = []
 
-    keys = DEVICE_ID_KEYS.values()
-    if filter_key:
-        keys = [filter_key]
-
     api_interface = WinkApiInterface()
 
+    keys = ['powerstrip_id', 'sensor_pod_id', 'piggy_bank_id',
+            'smoke_detector_id', 'hub_id']
+
     for item in items:
-        for key in keys:
-            if not __device_is_visible(item, key):
-                continue
-
-            if key == "powerstrip_id":
-                devices.extend(__get_outlets_from_powerstrip(item, api_interface))
-                continue  # Don't capture the powerstrip itself as a device, only the individual outlets
-
-            if key == "sensor_pod_id":
-                subsensors = _get_subsensors_from_sensor_pod(item, api_interface)
-                if subsensors:
-                    devices.extend(subsensors)
-                if len(subsensors) == 1:
-                    continue
-
+        if item.get(filter_key, None) is None:
+            continue
+        elif not __device_is_visible(item, filter_key):
+            continue
+        elif filter_key in keys:
+            devices.extend(__get_outlets_from_powerstrip(item, api_interface, filter_key))
+            devices.extend(__get_subsensors_from_sensor_pod(item, api_interface, filter_key))
+            devices.extend(__get_devices_from_piggy_bank(item, api_interface, filter_key))
+            devices.extend(__get_subsensors_from_smoke_detector(item, api_interface, filter_key))
+            devices.extend(__get_sensor_from_hub(item, api_interface, filter_key))
+        else:
             devices.append(build_device(item, api_interface))
 
     return devices
 
 
-def _get_subsensors_from_sensor_pod(item, api_interface):
+def __get_sensor_from_hub(item, api_interface, filter_key):
+    if filter_key != 'hub_id':
+        return []
+    keys = list(DEVICE_ID_KEYS.values())
+    # Most devices have a hub_id, but we only want the actual hub.
+    # This will only return hubs by checking for any other keys
+    # being present along with the hub_id
+    skip = False
+    for key in keys:
+        if key == "hub_id":
+            continue
+        if item.get(key, None) is not None:
+            skip = True
+    if skip:
+        return []
+    else:
+        return [WinkHub(item, api_interface)]
+
+
+def __get_subsensors_from_sensor_pod(item, api_interface, filter_key):
+    if filter_key != 'sensor_pod_id':
+        return []
 
     capabilities = [cap['field'] for cap in item.get('capabilities', {}).get('fields', [])]
+    capabilities.extend([cap['field'] for cap in item.get('capabilities', {}).get('sensor_types', [])])
+
     if not capabilities:
         return []
 
@@ -202,13 +300,24 @@ def _get_subsensors_from_sensor_pod(item, api_interface):
     if WinkLiquidPresenceSensor.CAPABILITY in capabilities:
         subsensors.append(WinkLiquidPresenceSensor(item, api_interface))
 
+    if WinkMotionSensor.CAPABILITY in capabilities:
+        subsensors.append(WinkMotionSensor(item, api_interface))
+
+    if WinkPresenceSensor.CAPABILITY in capabilities:
+        subsensors.append(WinkPresenceSensor(item, api_interface))
+
+    if WinkProximitySensor.CAPABILITY in capabilities:
+        subsensors.append(WinkProximitySensor(item, api_interface))
+
     if WinkSensorPod.CAPABILITY in capabilities:
         subsensors.append(WinkSensorPod(item, api_interface))
 
     return subsensors
 
 
-def __get_outlets_from_powerstrip(item, api_interface):
+def __get_outlets_from_powerstrip(item, api_interface, filter_key):
+    if filter_key != 'powerstrip_id':
+        return []
     outlets = item['outlets']
     for outlet in outlets:
         if 'subscription' in item:
@@ -217,6 +326,24 @@ def __get_outlets_from_powerstrip(item, api_interface):
     return [build_device(outlet, api_interface) for outlet in outlets if __device_is_visible(outlet, 'outlet_id')]
 
 
+def __get_devices_from_piggy_bank(item, api_interface, filter_key):
+    if filter_key != 'piggy_bank_id':
+        return []
+    subdevices = []
+    subdevices.append(WinkCurrencySensor(item, api_interface))
+    subdevices.append(WinkPorkfolioNose(item, api_interface))
+    return subdevices
+
+
+def __get_subsensors_from_smoke_detector(item, api_interface, filter_key):
+    if filter_key != 'smoke_detector_id':
+        return []
+    subsensors = []
+    subsensors.append(WinkSmokeDetector(item, api_interface))
+    subsensors.append(WinkCoDetector(item, api_interface))
+    return subsensors
+
+
 def __device_is_visible(item, key):
     is_correctly_structured = bool(item.get(key))
     is_visible = not item.get('hidden_at')
