@@ -1,14 +1,15 @@
 from pywink.devices.base import WinkDevice
-from pywink.devices.sensors import WinkSensorPod
+from pywink.devices.sensors import WinkSensorPod, WinkHub
 from pywink.devices.standard import WinkBulb, WinkBinarySwitch, WinkPowerStripOutlet, WinkLock, \
-    WinkEggTray, WinkGarageDoor, WinkShade, WinkSiren, WinkKey
+    WinkEggTray, WinkGarageDoor, WinkShade, WinkSiren, WinkKey, WinkThermostat, \
+    WinkFan
 
 
+# pylint: disable=redefined-variable-type,too-many-branches
 def build_device(device_state_as_json, api_interface):
 
     new_object = None
 
-    # pylint: disable=redefined-variable-type
     # These objects all share the same base class: WinkDevice
 
     if "light_bulb_id" in device_state_as_json:
@@ -31,5 +32,13 @@ def build_device(device_state_as_json, api_interface):
         new_object = WinkSiren(device_state_as_json, api_interface)
     elif "key_id" in device_state_as_json:
         new_object = WinkKey(device_state_as_json, api_interface)
+    elif "thermostat_id" in device_state_as_json:
+        new_object = WinkThermostat(device_state_as_json, api_interface)
+    elif "fan_id" in device_state_as_json:
+        new_object = WinkFan(device_state_as_json, api_interface)
+    # This must be at the bottom most devices have a hub_id listed
+    # as their associated hub.
+    elif "hub_id" in device_state_as_json:
+        new_object = WinkHub(device_state_as_json, api_interface)
 
     return new_object or WinkDevice(device_state_as_json, api_interface)
