@@ -8,6 +8,11 @@ POWER_STRIP = 'powerstrip'
 SHADE = 'shades'
 SIREN = 'siren'
 KEY = 'key'
+PIGGY_BANK = 'piggybank'
+SMOKE_DETECTOR = 'smoke_detector'
+THERMOSTAT = 'thermostat'
+HUB = 'hub'
+FAN = 'fan'
 
 DEVICE_ID_KEYS = {
     BINARY_SWITCH: 'binary_switch_id',
@@ -19,5 +24,10 @@ DEVICE_ID_KEYS = {
     SENSOR_POD: 'sensor_pod_id',
     SHADE: 'shade_id',
     SIREN: 'siren_id',
-    KEY: 'key_id'
+    KEY: 'key_id',
+    PIGGY_BANK: 'piggy_bank_id',
+    SMOKE_DETECTOR: 'smoke_detector_id',
+    THERMOSTAT: 'thermostat_id',
+    HUB: 'hub_id',
+    FAN: 'fan_id'
 }
